package org.exoplatform.services.cms.actions;

import java.util.ArrayList;
import java.util.Collection;
import java.util.HashMap;
import java.util.Iterator;
import java.util.List;
import java.util.Map;
import java.util.Set;

import javax.jcr.Node;
import javax.jcr.PathNotFoundException;
import javax.jcr.RepositoryException;
import javax.jcr.Session;
import javax.jcr.nodetype.NodeType;
import javax.jcr.nodetype.NodeTypeManager;
import javax.jcr.nodetype.PropertyDefinition;
import javax.jcr.observation.Event;
import javax.jcr.observation.ObservationManager;

import org.apache.commons.lang.StringUtils;
import org.exoplatform.services.cms.JcrInputProperty;
import org.exoplatform.services.jcr.core.ManageableRepository;

abstract public class BaseActionPlugin implements ActionPlugin {

  private Map<String, Session> sessions = new HashMap<String, Session>();

  protected Map<String, ECMEventListener> listeners_ = new HashMap<String, ECMEventListener>();

  abstract protected String getWorkspace();

  abstract protected ManageableRepository getRepository() throws Exception;

  abstract protected String getActionType();

  abstract protected List getActions();

  abstract protected void launchListener(ActionConfig.Action action,
      Node actionNode, Map variablesMap) throws Exception;

  abstract protected ECMEventListener createEventListener(String actionName,
      String actionExecutable, String srcWorkspace, String srcPath,
      Map variables) throws Exception;

  public void addAction(String actionType, String srcWorkspace, String srcPath,
      Map mappings) throws Exception {
    String actionName = (String) ((JcrInputProperty) mappings.get("/node/exo:name"))
        .getValue();
    mappings.remove("/node/exo:name");

    String type = (String) ((JcrInputProperty) mappings.get("/node/exo:lifecyclePhase"))
        .getValue();

    if (ActionServiceContainer.READ_PHASE.equals(type))
      return;

    Map variables = new HashMap();
    Set keys = mappings.keySet();
    for (Iterator iter = keys.iterator(); iter.hasNext();) {
      String key = (String) iter.next();
      Object value = ((JcrInputProperty) mappings.get(key)).getValue();
      key = key.substring(key.lastIndexOf("/") + 1);
      variables.put(key, value);
    }
    String actionExecutable = getActionExecutable(actionType);

    ECMEventListener listener = createEventListener(actionName, actionExecutable,
        srcWorkspace, srcPath, variables);
    ObservationManager obsManager = getSystemSession(srcWorkspace)
        .getWorkspace().getObservationManager();
    if (ActionServiceContainer.ADD_PHASE.equals(type)) {
      obsManager.addEventListener(listener, Event.NODE_ADDED, srcPath, true,
          null, null, false);
    } else {
      obsManager.addEventListener(listener, Event.NODE_REMOVED, srcPath, true,
          null, null, false);
    }
    listeners_.put(actionName, listener);
  }

  protected Session getSystemSession(String workspace) throws Exception {
    Session session = sessions.get(workspace);
    if (session == null) {
      ManageableRepository jcrRepository = getRepository();
      session = jcrRepository.getSystemSession(workspace);
      sessions.put(workspace, session);
    }
    return session;
  }

  public String getActionExecutable(String actionTypeName) throws Exception {
    Session session = null;
    try {
      session = getSystemSession(getWorkspace());
    } catch (RepositoryException ex) {
      return null;
    }

    NodeTypeManager ntManager = session.getWorkspace().getNodeTypeManager();
    NodeType nt = ntManager.getNodeType(actionTypeName);
    PropertyDefinition[] propDefs = nt.getDeclaredPropertyDefinitions();
    for (int i = 0; i < propDefs.length; i++) {
      PropertyDefinition definition = propDefs[i];
      if (definition.getName().equals(getExecutableDefinitionName())) {
        return definition.getDefaultValues()[0].getString();
      }
    }
    return null;
  }

  public boolean isActionTypeSupported(String actionType) {
    Session session = null;
    try {
      session = getSystemSession(getWorkspace());
      NodeTypeManager ntmanager = session.getWorkspace().getNodeTypeManager();
      NodeType[] superTypes = ntmanager.getNodeType(actionType).getSupertypes();
      for (int i = 0; i < superTypes.length; i++) {
        NodeType type = superTypes[i];
        if (getActionType().equals(type.getName()))
          return true;
      }
      return false;
    } catch (Exception re) {
      return false;
    }
  }

  public void removeAction(String actionName) throws Exception {
    ECMEventListener eventListener = (ECMEventListener) listeners_
        .get(actionName);
    String srcWorkspace = eventListener.getSrcWorkspace();
    ObservationManager obsManager = getSystemSession(srcWorkspace)
        .getWorkspace().getObservationManager();
    obsManager.removeEventListener(eventListener);
    listeners_.remove(actionName);
  }

  public boolean isVariable(String variable) throws Exception {
    Session session = null;
    try {
      session = getSystemSession(getWorkspace());
    } catch (RepositoryException ex) {
      return false;
    }

    NodeTypeManager ntManager = session.getWorkspace().getNodeTypeManager();
    NodeType nt = ntManager.getNodeType(getActionType());
    PropertyDefinition[] propDefs = nt.getDeclaredPropertyDefinitions();
    for (int i = 0; i < propDefs.length; i++) {
      PropertyDefinition definition = propDefs[i];
      if (definition.getName().equals(variable)) {
        return false;
      }
    }
    return true;
  }

  public Collection<String> getVariableNames(String actionTypeName)
      throws Exception {
    Collection<String> variableNames = new ArrayList<String>();

    Session session = null;
    try {
      session = getSystemSession(getWorkspace());
    } catch (RepositoryException ex) {
    }

    NodeTypeManager ntManager = session.getWorkspace().getNodeTypeManager();
    NodeType nt = ntManager.getNodeType(actionTypeName);
    PropertyDefinition[] propDefs = nt.getDeclaredPropertyDefinitions();
    for (int i = 0; i < propDefs.length; i++) {
      PropertyDefinition definition = propDefs[i];
      if (isVariable(definition.getName())) {
        variableNames.add(definition.getName());
      }
    }
    return variableNames;
  }

  protected void importPredefinedActionsInJcr() throws Exception {
    Session session = null;

    List actions = getActions();
    if (actions.isEmpty())
      return;

    for (Iterator iter = actions.iterator(); iter.hasNext();) {
      ActionConfig.Action action = (ActionConfig.Action) iter.next();
      String actionName = action.getName();
      String nodeType = action.getType();
      String srcWorkspace = action.getSrcWorkspace();
      String srcPath = action.getSrcPath();

      session = getSystemSession(srcWorkspace);
      try {
        Node srcNode = (Node) session.getItem(srcPath);

        Node actionNode = null;
        boolean firstImport = false;
        if (!srcNode.hasNode(actionName)) {
          firstImport = true;

          if (!srcNode.isNodeType("exo:actionable")) {
            srcNode.addMixin("exo:actionable");
          }

          actionNode = srcNode.addNode(actionName, nodeType);
          actionNode.setProperty("exo:name", action.getName());
          actionNode.setProperty("exo:description", action.getDescription());

          if (action.getLifecyclePhase() != null)
            actionNode.setProperty("exo:lifecyclePhase", action
                .getLifecyclePhase());
          if (action.getRoles() != null) {
            String[] roles = StringUtils.split(action.getRoles(), ";");
            actionNode.setProperty("exo:roles", roles);
          }

          Iterator mixins = action.getMixins().iterator();
          while (mixins.hasNext()) {
            ActionConfig.Mixin mixin = (ActionConfig.Mixin) mixins.next();
            actionNode.addMixin(mixin.getName());
            Map<String, String> props = mixin.getParsedProperties();
            Set keys = props.keySet();
            for (Iterator iterator = keys.iterator(); iterator.hasNext();) {
              String key = (String) iterator.next();
              actionNode.setProperty(key, props.get(key));
            }
          }
        } else {
          actionNode = srcNode.getNode(actionName);
        }

        String unparsedVariables = action.getVariables();
        Map variablesMap = new HashMap();
        if (unparsedVariables != null && !"".equals(unparsedVariables)) {
          String[] variables = StringUtils.split(unparsedVariables, ";");
          for (int i = 0; i < variables.length; i++) {
            String variable = variables[i];
            String[] keyValue = StringUtils.split(variable, "=");
            String variableKey = keyValue[0];
            String variableValue = keyValue[1];
            variablesMap.put(variableKey, variableValue);
            if (firstImport)
              actionNode.setProperty(variableKey, variableValue);
          }
        }
        if (firstImport)
          srcNode.save();

        if (action.getLifecyclePhase() != null) {
          if (ActionServiceContainer.ADD_PHASE.equals(action
              .getLifecyclePhase())
              || ActionServiceContainer.REMOVE_PHASE.equals(action
                  .getLifecyclePhase()))
            launchListener(action, actionNode, variablesMap);
        }

      } catch (PathNotFoundException e) {
      }
    }
  }

}
