/*
 * Created on Apr 3, 2005
 */
package org.exoplatform.services.cms.impl;

import java.io.ByteArrayInputStream;
import java.util.GregorianCalendar;
import java.util.Iterator;
import java.util.Map;
import java.util.Set;

import javax.jcr.Node;
import javax.jcr.NodeIterator;
import javax.jcr.PathNotFoundException;
import javax.jcr.Property;
import javax.jcr.PropertyIterator;
import javax.jcr.PropertyType;
import javax.jcr.Repository;
import javax.jcr.RepositoryException;
import javax.jcr.Session;
import javax.jcr.Value;
import javax.jcr.Workspace;
import javax.jcr.nodetype.NodeDefinition;
import javax.jcr.nodetype.NodeType;
import javax.jcr.nodetype.NodeTypeManager;
import javax.jcr.nodetype.PropertyDefinition;

import org.apache.commons.lang.StringUtils;
import org.exoplatform.commons.utils.ISO8601;
import org.exoplatform.container.PortalContainer;
import org.exoplatform.services.cms.CmsConfigurationService;
import org.exoplatform.services.cms.CmsService;
import org.exoplatform.services.cms.JcrInputProperty;
import org.exoplatform.services.idgenerator.IDGeneratorService;
import org.exoplatform.services.jcr.RepositoryService;
import org.exoplatform.services.jcr.core.ManageableRepository;
import org.exoplatform.services.jcr.datamodel.Uuid;
import org.exoplatform.services.jcr.impl.core.value.ReferenceValue;

/**
 * @author benjaminmestrallet
 */
public class CmsServiceImpl implements CmsService {
  
  private ThreadLocal draftSessions = new ThreadLocal();
  private ThreadLocal productionSessions = new ThreadLocal();
  
  private RepositoryService jcrService;

  private IDGeneratorService idGeneratorService;

  public CmsServiceImpl(RepositoryService jcrService,
      IDGeneratorService idGeneratorService) {
    this.idGeneratorService = idGeneratorService;
    this.jcrService = jcrService;
  }

  public String storeNode(String workspace, String nodeTypeName,
      String storePath, Map mappings) throws Exception {
    Repository repository = jcrService.getRepository();
    Session session = repository.login(workspace);
    Node storeHomeNode = (Node) session.getItem(storePath);
    String path = storeNode(nodeTypeName, storeHomeNode, mappings);
    storeHomeNode.save();
    return path;
  }

  public String storeNode(String nodeTypeName, Node storeHomeNode, Map mappings)
      throws Exception {
    Set keys = mappings.keySet();
    String nodePath = extractNodeName(keys);
    String nodeName = (String) ((JcrInputProperty) mappings.get(nodePath)).getValue();
    if (nodeName == null || "".equals(nodeName)) {
      nodeName = idGeneratorService.generateStringID(nodeTypeName);
    }

    Session session = storeHomeNode.getSession();
    NodeTypeManager nodetypeManager = session.getWorkspace()
        .getNodeTypeManager();
    NodeType nodeType = nodetypeManager.getNodeType(nodeTypeName);
    Node currentNode = null;
    if (storeHomeNode.hasNode(nodeName)) {
      currentNode = storeHomeNode.getNode(nodeName);
      updateNodeRecursively(NODE, currentNode, nodeType, mappings);
    } else {
      currentNode = storeHomeNode.addNode(nodeName, nodeTypeName);
      createNodeRecursively(NODE, currentNode, nodeType, mappings);
    }
    return currentNode.getPath();
  }

  private void updateNodeRecursively(String path, Node currentNode,
      NodeType currentNodeType, Map jcrVariables) throws Exception {
    processNodeRecursively(false, path, currentNode, currentNodeType,
        jcrVariables);
  }

  private void createNodeRecursively(String path, Node currentNode,
      NodeType currentNodeType, Map jcrVariables) throws Exception {
    processNodeRecursively(true, path, currentNode, currentNodeType,
        jcrVariables);
  }

  private void processNodeRecursively(boolean create, String path,
      Node currentNode, NodeType currentNodeType, Map jcrVariables)
      throws Exception {

    PropertyDefinition[] propertyDefs = currentNodeType
        .getPropertyDefinitions();

    for (int i = 0; i < propertyDefs.length; i++) {
      PropertyDefinition propertyDef = propertyDefs[i];
      if (!propertyDef.isAutoCreated() && !propertyDef.isProtected()) {
        String propertyName = propertyDef.getName();
        int requiredtype = propertyDef.getRequiredType();
        String currentPath = path + "/" + propertyName;
        JcrInputProperty inputVariable = (JcrInputProperty) jcrVariables.get(currentPath);
        if (propertyDef.isMandatory()) {
          createProperty(propertyName, currentNode, requiredtype, inputVariable.getValue());
        } else {
          if (inputVariable != null && inputVariable.getValue() != null)
            createProperty(propertyName, currentNode, requiredtype, inputVariable.getValue());
        }
      }
    }
    NodeDefinition[] childNodeDefs = currentNodeType.getChildNodeDefinitions();
    for (int i = 0; i < childNodeDefs.length; i++) {
      NodeDefinition nodeDef = childNodeDefs[i];
      if (!nodeDef.isAutoCreated() && !nodeDef.isProtected()) {
        String nodeName = nodeDef.getName();
        String currentPath = path + "/" + nodeName;
        JcrInputProperty inputVariable = (JcrInputProperty) jcrVariables.get(currentPath);
        String nodetypeName = inputVariable.getNodetype();
        NodeType nodeType = null;
        if(nodetypeName == null || "".equals(nodetypeName)){
          nodeType = nodeDef.getRequiredPrimaryTypes()[0];          
        } else {
          Session session = currentNode.getSession();
          NodeTypeManager nodetypeManager = session.getWorkspace()
              .getNodeTypeManager();
          nodeType = nodetypeManager.getNodeType(nodetypeName);
        }
        Node childNode = null;
        if (create) {
          childNode = currentNode.addNode(nodeName, nodeType.getName());
        } else {
          try {
            childNode = currentNode.getNode(nodeName);
          } catch (PathNotFoundException e) {
            childNode = currentNode.addNode(nodeName, nodeType.getName());
          }
        }
        processNodeRecursively(create, path + "/" + nodeName, childNode,
            nodeType, jcrVariables);
      }
    }
  }

  private void createProperty(String propertyName, Node node, int requiredtype,
      Object value) throws Exception {
    switch (requiredtype) {
    case PropertyType.STRING:
      if (value == null)
        node.setProperty(propertyName, "");
      else 
        node.setProperty(propertyName, (String) value);
      break;
    case PropertyType.BINARY:
      if (value == null)
        node.setProperty(propertyName, "");
      else if (value instanceof byte[])
        node.setProperty(propertyName, 
            new ByteArrayInputStream((byte[]) value));
      else if (value instanceof String)
        node.setProperty(propertyName, (String) value);
      break;
    case PropertyType.BOOLEAN:
      if (value == null)
        node.setProperty(propertyName, false);
      else 
        node.setProperty(propertyName, 
          new Boolean((String) value).booleanValue());
      break;
    case PropertyType.LONG:
      if (value == null)
        node.setProperty(propertyName, 0);
      else 
        node.setProperty(propertyName, new Long((String) value).longValue());
      break;
    case PropertyType.DOUBLE:
      if (value == null)
        node.setProperty(propertyName, 0);
      else
        node.setProperty(propertyName, new Double((String) value).doubleValue());
      break;
    case PropertyType.DATE:
      if (value == null)
        node.setProperty(propertyName, new GregorianCalendar());
      if (value instanceof String) {
        node.setProperty(propertyName, ISO8601.parse((String) value));
      } else if (value instanceof GregorianCalendar) {
        node.setProperty(propertyName, (GregorianCalendar) value);
      }
      break;
    case PropertyType.REFERENCE:
    	if (value == null)
    		throw new RepositoryException("null value for a reference " + requiredtype);
    	if (value instanceof Value[]) 
    		node.setProperty(propertyName, (Value[]) value);
      else if (value instanceof String)
    		node.setProperty(propertyName, (String) value);
    	break ;
    default:
      throw new RepositoryException("unknown type " + requiredtype);
    }
  }

  private String extractNodeName(Set keys) {
    for (Iterator iter = keys.iterator(); iter.hasNext();) {
      String key = (String) iter.next();
      if (key.endsWith(NODE))
        return key;
    }
    return null;
  }
  
  public void moveNode(String nodePath, String srcWorkspace, String destWorkspace,
      String destPath) {
    PortalContainer container = PortalContainer.getInstance();
    RepositoryService repService = (RepositoryService) container
        .getComponentInstanceOfType(RepositoryService.class);
    
    try {
      ManageableRepository repository = repService.getRepository();
      Session srcSession = repository.getSystemSession(srcWorkspace);
      Session destSession = repository.getSystemSession(destWorkspace);
      Workspace workspace = destSession.getWorkspace();
      Node srcNode = (Node) srcSession.getItem(nodePath);
      try {
        Node permNode = (Node) destSession.getItem(destPath);        
        updateNodeWithNode(permNode, srcNode);
        destSession.save();                
      } catch (PathNotFoundException e) {
        createNode(destSession, destPath);
        destSession.save();
        workspace.clone(srcWorkspace, nodePath, destPath, true);
      }
      //Remove src node
      srcNode.remove();
      srcSession.save();
    } catch (Exception e) {
      e.printStackTrace();
    }
  }

  private void updateNodeWithNode(Node permNode, Node tempNode)
      throws Exception {
    PropertyIterator properties = permNode.getProperties();
    while (properties.hasNext()) {
      Property property = properties.nextProperty();
      PropertyDefinition propDef = property.getDefinition();
      if (!propDef.isAutoCreated() && !propDef.isProtected()) {
        String name = property.getName();
        Value newValue = tempNode.getProperty(name).getValue();
        property.setValue(newValue);
      }
    }
    
    //TODO ADD THE NEW MIXIN TYPES TOOOOOO

    NodeIterator nodes = permNode.getNodes();
    while (nodes.hasNext()) {
      Node node = nodes.nextNode();
      String name = node.getName();
      updateNodeWithNode(node, tempNode.getNode(name));
    }
  }

  private void createNode(Session session, String uri) throws RepositoryException {
    String[] splittedName = StringUtils.split(uri, "/");    
    Node node = session.getRootNode();
    for (int i = 0; i < splittedName.length - 2; i++) {
      try {
        node = node.getNode(splittedName[i]);
      } catch (PathNotFoundException exc) {
        node = node.addNode(splittedName[i], "nt:folder");
      }
    }
    try {
      node = node.getNode(splittedName[splittedName.length - 2]);
    } catch (PathNotFoundException exc) {
   		node = node.addNode(splittedName[splittedName.length - 2], "nt:file");
    }
  }

  public Session getSession(String workspace) throws Exception{
    Session session = (Session) draftSessions.get();
    if(session == null) {
      Repository repository = jcrService.getRepository();
      if(workspace == null)
        session = repository.login();
      else          
        session = repository.login(workspace);
      draftSessions.set(session);
    }
    return session;
  }
}
