/*
 * Copyright 2005 Day Management AG, Switzerland. All rights reserved.
 */
package javax.jcr.nodetype;

import javax.jcr.Value;

/**
 * Represents a node type.
 *
 */
public interface NodeType {

    /**
     * Returns the name of the node type.
     *
     * @return the name of the node type
     */
    public String getName();

    /**
     * Returns <code>true</code> if this node type is a mixin node type.
     * Returns <code>false</code> if this node type is a primary node type.
     *
     * @return a boolean
     */
    public boolean isMixin();

    /**
     * Returns <code>true</code> if nodes of this type must support orderable child nodes; returns <code>false</code>
     * otherwise. If a node type returns <code>true</code> on a call to this method, then all nodes of that node type
     * <i>must</i> support the method {@link javax.jcr.Node#orderBefore}. If a node type returns <code>false</code>
     * on a call to this method, then nodes of that node type <i>may</i> support these ordering methods. Only the primary
     * node type of a node controls that node's status in this regard. This setting on a mixin node type will not have any effect
     * on the node.
     *
     * @return Returns <code>true</code> if nodes of this type must support orderable child nodes; returns
     * <code>false</code> otherwise.
     */
    public boolean hasOrderableChildNodes();

    /**
     * Returns the name of the primary item (one of the child items of the nodes of this node type).
     * If this node has no primary item, then this method returns <code>null</code>.
     * This indicator is used by the method {@link javax.jcr.Node#getPrimaryItem()}.
     *
     * @return the name of the primary item.
     */
    public String getPrimaryItemName();

    /**
     * Returns all supertypes of this node type in the node type inheritance hierarchy.
     * For primary types apart from <code>nt:base</code>, this list will always include
     * at least <code>nt:base</code>. For mixin types, there is no required supertype.
     *
     * @see #getDeclaredSupertypes
     *
     * @return an array of <code>NodeType</code> objects.
     */
    public NodeType[] getSupertypes();

    /**
     * Returns the <i>direct</i> supertypes of this node type in the node type inheritance hierarchy,
     * that is, those actually declared in this node type. In single-inheritance systems, this
     * will always be an array of size 0 or 1. In systems that support multiple inheritance of
     * node types this array may be of size greater than 1.
     *
     * @see #getSupertypes
     *
     * @return an array of <code>NodeType</code> objects.
     */
    public NodeType[] getDeclaredSupertypes();

    /**
     * Returns true if this node type is <code>nodeTypeName</code>
     * or a subtype of <code>nodeTypeName</code>, otherwise returns
     * <code>false</code>.
     * @param nodeTypeName the name of a node type.
     * @return a boolean
     */
    public boolean isNodeType(String nodeTypeName);

    /**
     * Returns an array containing the property definitions of this node type,
     * including the property definitions inherited from supertypes of this node
     * type.
     *
     * @see #getDeclaredPropertyDefinitions
     *
     * @return an array containing the property definitions.
     */
    public PropertyDefinition[] getPropertyDefinitions();

    /**
     * Returns an array containing the property definitions explicitly specified
     * in the declaration of <i>this</i> node type. This does <i>not</i> include
     * property definitions inherited from supertypes of this node type.
     *
     * @see #getPropertyDefinitions
     *
     * @return an array containing the property definitions.
     */
    public PropertyDefinition[] getDeclaredPropertyDefinitions();

    /**
     * Returns an array containing the child node definitions of this node type,
     * including the child node definitions inherited from supertypes of this
     * node type.
     *
     * @see #getDeclaredChildNodeDefinitions
     *
     * @return an array containing the child node definitions.
     */
    public NodeDefinition[] getChildNodeDefinitions();

    /**
     * Returns an array containing the child node definitions explicitly
     * specified in the declaration of <i>this</i> node type. This does
     * <i>not</i> include child node definitions inherited from supertypes of
     * this node type.
     *
     * @see #getChildNodeDefinitions
     * @return an array containing the child node definitions.
     */
    public NodeDefinition[] getDeclaredChildNodeDefinitions();

    /**
     * Returns <code>true</code> if setting <code>propertyName</code> to
     * <code>value</code> is allowed by this node type. Otherwise returns
     * <code>false</code>.
     *
     * @param propertyName The name of the property
     * @param value A <code>Value</code> object.
     */
    public boolean canSetProperty(String propertyName, Value value);

    /**
     * Returns <code>true</code> if setting <code>propertyName</code> to
     * <code>values</code> is allowed by this node type. Otherwise returns
     * <code>false</code>.
     *
     * @param propertyName The name of the property
     * @param values A <code>Value</code> array.
     */
    public boolean canSetProperty(String propertyName, Value[] values);

    /**
     * Returns <code>true</code> if this node type allows the addition of a
     * child node called <code>childNodeName</code> without specific node type
     * information (that is, given the definition of this parent node type, the
     * child node name is sufficient to determine the intended child node type).
     * Returns <code>false</code> otherwise.
     *
     * @param childNodeName The name of the child node.
     */
    public boolean canAddChildNode(String childNodeName);

    /**
     * Returns <code>true</code> if this node type allows the addition of a
     * child node called <code>childNodeName</code> of node type
     * <code>nodeTypeName</code>. Returns <code>false</code> otherwise.
     *
     * @param childNodeName The name of the child node.
     * @param nodeTypeName The name of the node type of the child node.
     */
    public boolean canAddChildNode(String childNodeName, String nodeTypeName);

    /**
     * Returns true if removing the child item called <code>itemName</code> is allowed by this node type.
     * Otherwise returns <code>false</code>.
     *
     * @param itemName The name of the child item
     */
    public boolean canRemoveItem(String itemName);
}