/***************************************************************************
 * Copyright 2001-2003 The eXo Platform SARL         All rights reserved.  *
 * Please look at license.txt in info directory for more license detail.   *
 **************************************************************************/

package org.exoplatform.services.jcr.impl.core;

import java.util.Collection;
import java.util.HashMap;
import java.util.Iterator;
import java.util.Map;
import java.util.Set;

import javax.jcr.ItemExistsException;
import javax.jcr.NamespaceException;
import javax.jcr.NamespaceRegistry;
import javax.jcr.Node;
import javax.jcr.NodeIterator;
import javax.jcr.PathNotFoundException;
import javax.jcr.Property;
import javax.jcr.PropertyType;
import javax.jcr.RepositoryException;
import javax.jcr.Value;

import org.apache.commons.lang.ArrayUtils;
import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.exoplatform.commons.Environment;
import org.exoplatform.services.jcr.access.AccessControlPolicy;
import org.exoplatform.services.jcr.core.NamespaceAccessor;
import org.exoplatform.services.log.LogService;

/**
 * Created by The eXo Platform SARL .
 * 
 * @author <a href="mailto:geaz@users.sourceforge.net">Gennady Azarenkov </a>
 * @version $Id: NamespaceRegistryImpl.java 3611 2006-01-04 11:46:01Z peterit $
 */

public class NamespaceRegistryImpl implements NamespaceRegistry,
    NamespaceAccessor {

  public static HashMap DEF_NAMESPACES = new HashMap();

  private static String NS_ROOT = "/jcr:system/exo:namespaces";

  private static String NS_ROOT_URI = "/jcr:system/exo:namespaces";

  static {
    DEF_NAMESPACES.put("", "");
    DEF_NAMESPACES.put("jcr", "http://www.jcp.org/jcr/1.0");
    DEF_NAMESPACES.put("nt", "http://www.jcp.org/jcr/nt/1.0");
    DEF_NAMESPACES.put("mix", "http://www.jcp.org/jcr/mix/1.0");
    DEF_NAMESPACES.put("sv", "http://www.jcp.org/jcr/sv/1.0");
    DEF_NAMESPACES.put("exo", "http://www.exoplatform.com/jcr/exo/1.0");
  }

  private HashMap namespaces;

  protected Log log;

  private SessionImpl systemSession = null;

  //private NodeImpl nsRoot = null;

  private static final String[] protectedNamespaces = { "jcr", "nt", "mix",
      "sv", "exo" };

  public NamespaceRegistryImpl(LogService logService)
      throws RepositoryException {
    log = logService.getLog("org.exoplatform.services.jcr");
    //repository guarantees default set of namespaces
    namespaces = DEF_NAMESPACES;
  }

  // for testing
  public NamespaceRegistryImpl() throws RepositoryException {
    log = LogFactory.getLog("org.exoplatform.services.jcr");
    //repository guarantees default set of namespaces
    namespaces = DEF_NAMESPACES;
  }

  /**
   * It is impossible to inject persister as namespaces should be instantiated
   * first (persister should use at least default set of namespaces)
   * 
   * @param persister
   */
  public void setSystemSession(SessionImpl systemSession) {
    this.systemSession = systemSession;
  }

  /**
   * @see javax.jcr.NamespaceRegistry#getURI
   */
  public String getURI(String prefix) throws NamespaceException {
    String uri = (String) namespaces.get(prefix);
    if (uri == null)
      throw new NamespaceException("Unknown Prefix " + prefix);
    return uri;
  }

  /**
   * @see javax.jcr.NamespaceRegistry#registerNamespace
   */
  public synchronized void registerNamespace(String prefix, String uri)
      throws NamespaceException, RepositoryException {
    
    validateNamespace(prefix, uri);
    
    Collection values = namespaces.values();
    if (values.contains(uri)) {
      /*
      String key2Remove = null;
      Set keys = namespaces.keySet();
      for (Iterator iterator = keys.iterator(); iterator.hasNext();) {
        String key = (String) iterator.next();
        String value = (String) namespaces.get(key);
        if (value.equals(uri)) {
          key2Remove = key;
          break;
        }
      }
      namespaces.remove(key2Remove);
      delete(key2Remove);
      */
      throw new NamespaceException(
        "Re-registration is not supported as may cause integrity problems. (todo issue #46)");
    }
    add(prefix, uri);
    systemSession.save();
    namespaces.put(new String(prefix), new String(uri));

  }
  
  public synchronized void validateNamespace(String prefix, String uri)
      throws NamespaceException, RepositoryException {

    if (ArrayUtils.contains(protectedNamespaces, prefix)) {
      if (uri == null)
        throw new NamespaceException("Can not remove built-in namespace");
      else
        throw new NamespaceException("Can not change built-in namespace");
    }
    if (prefix.toLowerCase().startsWith("xml"))
      throw new NamespaceException(
          "Can not re-assign prefix that start with 'xml'");
    if (uri == null)
      throw new NamespaceException("Can not register NULL URI!");
  }  

  /**
   * @see javax.jcr.NamespaceRegistry#unregisterNamespace
   */
  public synchronized void unregisterNamespace(String prefix) throws NamespaceException,
      RepositoryException {

    //if (Environment.getInstance().getPlatform() != Environment.STAND_ALONE)
    //  throw new NamespaceException(
    //      "Currently Unregistration is supported for the testing purpose only as may cause integrity problems.(checking if there are nodes of this node type should be involved!)");

    if (namespaces.get(prefix) == null)
      throw new NamespaceException("Prefix " + prefix + " is not registered");

    for (int i = 0; i < protectedNamespaces.length; i++)
      if (prefix.equals(protectedNamespaces[i]))
        throw new NamespaceException("Prefix " + prefix + " is not protected");
      
    throw new NamespaceException(
      "Unregistration is not supported as may cause integrity problems. (todo issue #46)");
  
    //delete(prefix);
    //systemSession.save();
    //namespaces.remove(prefix);
  }

  /**
   * @see javax.jcr.NamespaceRegistry#getPrefixes
   */
  public String[] getPrefixes() {
    return (String[]) namespaces.keySet().toArray(
        new String[namespaces.keySet().size()]);
  }

  /**
   * @see javax.jcr.NamespaceRegistry#getURIs
   */
  public String[] getURIs() {
    return (String[]) namespaces.values()
        .toArray(new String[namespaces.size()]);
  }

  /**
   * @see javax.jcr.NamespaceRegistry#getPrefix
   */
  public String getPrefix(String uri) throws NamespaceException,
      RepositoryException {
    String[] prefixes = getPrefixes();
    for (int i = 0; i < prefixes.length; i++) {
      if (getURI(prefixes[i]).equals(uri))
        return prefixes[i];
    }
    throw new NamespaceException("Prefix for " + uri + " not found");
  }

  public Map getURIMap() {
    return namespaces;
  }

  ////////////////////// NamespaceAccessor
  public String getNamespaceURIByPrefix(String prefix)
      throws NamespaceException {
    return getURI(prefix);
  }

  public String getNamespacePrefixByURI(String uri) throws NamespaceException,
      RepositoryException {
    return getPrefix(uri);
  }

  public String[] getAllNamespacePrefixes() {
    return getPrefixes();
  }

  ///////////////////////////////////////

  /**
   * Saves namespace in persistent storage
   * 
   * @param prefix
   * @param uri
   * @throws RepositoryException
   * @throws ItemExistsException
   */
  private void add(String prefix, String uri) throws RepositoryException,
      ItemExistsException {

    if (systemSession == null) {
      log.debug("Session for persistence management should have been initialized. Default namespaces only will be used ");
      return;
    }

    NodeImpl ns = getNsRoot().createChildNode(prefix, "exo:namespace", false,
        true);
    Value[] values = new Value[] { systemSession.getValueFactory().createValue(
        uri) };
    Property uriProp = ns.createChildProperty("exo:uri", values,
        PropertyType.STRING);
    values = new Value[] { systemSession.getValueFactory().createValue(prefix) };
    Property prefixProp = ns.createChildProperty("exo:prefix", values,
        PropertyType.STRING);

  }

  private void delete(String prefix) throws RepositoryException {

    if (systemSession == null) {
      log.debug("Session for persistence management should have been initialized. Default namespaces only will be used ");
      return;
    }

    Node nsNode = getNsRoot().getNode(prefix);
    nsNode.remove();
  }

  private NodeImpl getNsRoot() throws RepositoryException,
      PathNotFoundException {
    return (NodeImpl) systemSession.getItem(NS_ROOT);
  }

  public void loadFromStorage() throws RepositoryException {

    if (systemSession == null) {
      log
          .debug("Session for persistence management should have been initialized. Default namespaces only will be used ");
      return;
    }
    NodeIterator nodes;
    try {
      nodes = getNsRoot().getNodes();
    } catch (PathNotFoundException e) {
      //e.printStackTrace();
      log
          .info("Namespaces storage (/jcr:system/exo:namespaces) is not accessible Default namespaces only will be used ");
      return;
    }
    while (nodes.hasNext()) {
      Node ns = nodes.nextNode();
      if (ns.getPrimaryNodeType().isNodeType("exo:namespace")) {
        String uri = ns.getProperty("exo:uri").getString();
        String prefix = ns.getProperty("exo:prefix").getString();

        namespaces.put(prefix, uri);
        log.info("Namespace " + prefix + " is loaded");
      }
    }
  }

  void initStorage(Node sysNode, String accessControlPolicy) throws RepositoryException {

    System.out.println("NS manager set>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>> " +
    		" "+accessControlPolicy+" "+accessControlPolicy.equals(AccessControlPolicy.DISABLE));

    if (systemSession == null)
      throw new RepositoryException(
          "System session should have been initialized");

    // initialize /jcr:system/exo:namespaces node
    try {
      NodeImpl nsRoot = (NodeImpl) sysNode.addNode("exo:namespaces", "nt:unstructured");
      
      if (!accessControlPolicy.equals(AccessControlPolicy.DISABLE)) { 
        //nsRoot.doAddMixin("exo:accessControllable", true, true);
        nsRoot.doAddMixin("exo:accessControllable", true, false, true);
      }
    } catch (ItemExistsException e) {
      // already initialized
    }

    Iterator i = namespaces.keySet().iterator();
    while (i.hasNext()) {
      String key = new String((String) i.next());
      try {
        log.debug("Namespace " + key + " " + (String) namespaces.get(key));
        if (!key.equals(""))
          add(key, (String) namespaces.get(key));
      } catch (ItemExistsException e) {
      }
      log.info("Namespace " + key + " is initialized.");
    }

  }

}