/*
 * Copyright 2004-2005 The Apache Software Foundation or its licensors,
 *                     as applicable.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package org.exoplatform.services.jcr.impl.core.query;

import javax.jcr.ItemExistsException;
import javax.jcr.ItemNotFoundException;
import javax.jcr.Node;
import javax.jcr.PathNotFoundException;
import javax.jcr.PropertyType;
import javax.jcr.RepositoryException;
import javax.jcr.UnsupportedRepositoryOperationException;
import javax.jcr.Value;
import javax.jcr.lock.LockException;
import javax.jcr.nodetype.ConstraintViolationException;
import javax.jcr.query.InvalidQueryException;
import javax.jcr.query.Query;
import javax.jcr.query.QueryResult;
import javax.jcr.version.VersionException;

import org.exoplatform.services.jcr.impl.core.JCRPath;
import org.exoplatform.services.jcr.impl.core.NodeImpl;
import org.exoplatform.services.jcr.impl.core.SessionImpl;

/**
 * Implements the {@link Query} interface.
 */
public class QueryImpl implements Query {

    /**
     * The session of the user executing this query
     */
    private final SessionImpl session;

    /**
     * The query statement
     */
    private final String statement;

    /**
     * The syntax of the query statement
     */
    private final String language;

    /**
     * The actual query implementation that can be executed
     */
    private final ExecutableQuery query;

    /**
     * The node where this query is persisted. Only set when this is a persisted
     * query.
     */
    private Node node;

    /**
     * Creates a new query instance from a query string.
     *
     * @param session   the session of the user executing this query.
     * @param itemMgr   the item manager of the session executing this query.
     * @param handler   the query handler of the search index.
     * @param statement the query statement.
     * @param language  the syntax of the query statement.
     * @throws InvalidQueryException if the query statement is invalid according
     *                               to the specified <code>language</code>.
     */
    public QueryImpl(SessionImpl session,
                     QueryHandler handler,
                     String statement,
                     String language) throws InvalidQueryException {
        this.session = session;
        this.statement = statement;
        this.language = language;
        this.query = handler.createExecutableQuery(session, statement, language);
    }

    /**
     * Create a new query instance from a nt:query node.
     *
     * @param session the session of the user executing this query.
     * @param itemMgr the item manager of the session executing this query.
     * @param handler the query handler of the search index.
     * @param node    a node of type <code>nt:query</code>.
     * @throws InvalidQueryException If <code>node</code> is not a valid persisted query
     *                               (that is, a node of type <code>nt:query</code>).
     * @throws RepositoryException   if another error occurs
     */
    public QueryImpl(SessionImpl session,
                     QueryHandler handler,
                     Node node)
            throws InvalidQueryException, RepositoryException {

        this.session = session;
        this.node = node;

            if (!node.isNodeType("nt:query")) {
                throw new InvalidQueryException("node is not of type nt:query");
            }
            statement = node.getProperty("jcr:statement").getString();
            language = node.getProperty("jcr:language").getString();
            query = handler.createExecutableQuery(session, statement, language);
    }

    /**
     * This method simply forwards the <code>execute</code> call to the
     * {@link ExecutableQuery} object returned by
     * {@link QueryHandler#createExecutableQuery}.
     * {@inheritDoc}
     */
    public QueryResult execute() throws RepositoryException {
        return query.execute();
    }

    /**
     * {@inheritDoc}
     */
    public String getStatement() {
        return statement;
    }

    /**
     * {@inheritDoc}
     */
    public String getLanguage() {
        return language;
    }

    /**
     * {@inheritDoc}
     */
    public String getStoredQueryPath()
            throws ItemNotFoundException, RepositoryException {
        if (node == null) {
            throw new ItemNotFoundException("not a persistent query");
        }
        return node.getPath();
    }

    /**
     * {@inheritDoc}
     */
    public Node storeAsNode(String absPath)
            throws ItemExistsException,
            PathNotFoundException,
            VersionException,
            ConstraintViolationException,
            LockException,
            UnsupportedRepositoryOperationException,
            RepositoryException {
      
      JCRPath path = session.getLocationFactory().parseAbsPath(absPath);
      String parentPath = path.makeParentPath().getAsString(false);
      String nodeName = path.getName().getAsString();
      
      NodeImpl parent = (NodeImpl)session.getItem(parentPath);
      
      NodeImpl queryNode = parent.createChildNode(nodeName, "nt:query", false, true);
      
      // set properties
      Value[] vals = new Value[] {session.getValueFactory().createValue(language)};
      queryNode.createChildProperty("jcr:language", vals, PropertyType.STRING);
      vals = new Value[] {session.getValueFactory().createValue(statement)};
      queryNode.createChildProperty("jcr:statement", vals, PropertyType.STRING);
      node = queryNode;
      System.out.println("Store as node: "+node.getPath()+" "+session+"  parent "+parent);
      return node;
        
    }
}

