/*
 * Copyright 2005 Day Management AG, Switzerland. All rights reserved.
 */
package org.exoplatform.services.jcr.impl.core.value;

import java.util.Calendar;

import javax.jcr.PropertyType;
import javax.jcr.RepositoryException;
import javax.jcr.Value;
import javax.jcr.ValueFormatException;

import org.exoplatform.services.jcr.datamodel.IllegalNameException;
import org.exoplatform.services.jcr.datamodel.InternalQName;
import org.exoplatform.services.jcr.datamodel.ValueData;
import org.exoplatform.services.jcr.impl.core.JCRName;
import org.exoplatform.services.jcr.impl.core.LocationFactory;
import org.exoplatform.services.jcr.impl.storage.TransientValueData;

/**
 * A <code>NameValue</code> provides an implementation of the
 * <code>Value</code> interface representing a <code>NAME</code> value (a
 * string that is namespace-qualified).
 * 
 * @author Stefan Guggisberg
 */
public class NameValue extends BaseValue {

  public static final int TYPE = PropertyType.NAME;

  private final JCRName name;

  /**
   * Protected constructor creating a <code>NameValue</code> object without
   * validating the name.
   * 
   * @param name
   *          the name this <code>NameValue</code> should represent
   * @see #valueOf
   */
  NameValue(JCRName name) {
    super(TYPE, new TransientValueData(name.getInternalName()));
    this.name = name;
  }

  NameValue(ValueData data, LocationFactory locationFactory)
      throws RepositoryException {
    super(TYPE, data);
    InternalQName qname;
    try {
      qname = InternalQName.parse(new String(data
          .getAsByteArray()));
    } catch (IllegalNameException e) {
      throw new RepositoryException(e);
    }
    this.name = locationFactory.createJCRName(qname);

  }
  
  public InternalQName getQName() {
    return name.getInternalName();
  }

  /**
   * Indicates whether some other object is "equal to" this one. <p/>The result
   * is <code>true</code> if and only if the argument is not <code>null</code>
   * and is a <code>NameValue</code> object that represents the same value as
   * this object.
   * 
   * @param obj
   *          the reference object with which to compare.
   * @return <code>true</code> if this object is the same as the obj argument;
   *         <code>false</code> otherwise.
   */
  public boolean equals(Object obj) {
    if (this == obj) {
      return true;
    }
    if (obj instanceof NameValue) {
      NameValue other = (NameValue) obj;
      if (name == other.name) {
        return true;
      } else if (name != null && other.name != null) {
        return name.equals(other.name);
      }
    }
    return false;
  }

  /**
   * @see BaseValue#getInternalString
   */
  protected String getInternalString() throws ValueFormatException {
    if (name != null) {
      return name.getAsString();
    } else {
      throw new ValueFormatException("empty value");
    }
  }

  //----------------------------------------------------------------< Value >
  /**
   * @see Value#getDate
   */
  public Calendar getDate() throws ValueFormatException, IllegalStateException,
      RepositoryException {
    setValueConsumed();

    throw new ValueFormatException(
        "conversion to date failed: inconvertible types");
  }

  /**
   * @see Value#getLong
   */
  public long getLong() throws ValueFormatException, IllegalStateException,
      RepositoryException {
    setValueConsumed();

    throw new ValueFormatException(
        "conversion to long failed: inconvertible types");
  }

  /**
   * @see Value#getBoolean
   */
  public boolean getBoolean() throws ValueFormatException,
      IllegalStateException, RepositoryException {
    setValueConsumed();

    throw new ValueFormatException(
        "conversion to boolean failed: inconvertible types");
  }

  /**
   * @see Value#getDouble
   */
  public double getDouble() throws ValueFormatException, IllegalStateException,
      RepositoryException {
    setValueConsumed();

    throw new ValueFormatException(
        "conversion to double failed: inconvertible types");
  }
}