/**
 * Copyright 2001-2003 The eXo Platform SARL         All rights reserved.  *
 * Please look at license.txt in info directory for more license detail.   *
 */

package org.exoplatform.services.jcr.impl.storage.cache;

import java.util.Iterator;
import java.util.List;

import javax.jcr.RepositoryException;

import org.exoplatform.services.cache.CacheService;
import org.exoplatform.services.cache.ExoCache;
import org.exoplatform.services.jcr.config.CacheEntry;
import org.exoplatform.services.jcr.config.WorkspaceEntry;
import org.exoplatform.services.jcr.datamodel.IllegalPathException;
import org.exoplatform.services.jcr.datamodel.InternalQPath;
import org.exoplatform.services.jcr.datamodel.ItemData;
import org.exoplatform.services.jcr.datamodel.NodeData;
import org.exoplatform.services.jcr.datamodel.PropertyData;
import org.exoplatform.services.jcr.impl.core.ItemChangeState;
import org.exoplatform.services.jcr.impl.core.ItemState;
import org.exoplatform.services.jcr.impl.core.ItemsPersistenceListener;
import org.exoplatform.services.jcr.impl.core.SessionImpl;
import org.exoplatform.services.log.LogService;
import org.exoplatform.services.log.impl.ExoLog;

/**
 * Created by The eXo Platform SARL .
 * 
 * @author <a href="mailto:gennady.azarenkov@exoplatform.com">Gennady Azarenkov
 *         </a>
 * @version $Id: WorkspaceCache.java 3547 2006-01-03 08:42:56Z geaz $
 */

public class WorkspaceCache implements ItemsPersistenceListener{

  protected ExoLog log;

  private ExoCache cache;

  private String name;

  private boolean enabled;

  public WorkspaceCache(
      CacheService cacheService,
      WorkspaceEntry wsConfig,
      LogService logService) throws Exception {

    this.log = (ExoLog)logService.getLog(this.getClass());
    
    name = "jcr." + wsConfig.getUniqueName();
    cache = cacheService.getCacheInstance(name);
    
    CacheEntry cacheConfig = wsConfig.getCache();
    if(cacheConfig != null) {
      enabled = cacheConfig.isEnabled();
      cache.setMaxSize(Integer.parseInt(cacheConfig.getParameterValue("maxSize")));
      cache.setLiveTime(Long.parseLong(cacheConfig.getParameterValue("liveTime")));
    } else {
      enabled = true;
    }
  }
  
  
  /**
   * @param name
   * @return
   * @throws Exception
   */
  public ItemData get(InternalQPath path) {
    log.setLevel(LogService.INFO);

    if (!enabled)
      return null;

    try {
      //log.debug("Cache get by path >> "+path.getAsString()+" uuid "+);
      ItemData data = (ItemData) cache.get(path.getAsString());
      if (data != null && data instanceof NodeData)
        log.debug("Cache get by path >> " + path.getAsString()
            + " found uuid: " + data.getUUID() + " primaryType: "
            + ((NodeData) data).getPrimaryTypeName().getAsString());

      return data;
    } catch (Exception e) {
      return null;
    }
  }

  public ItemData get(String uuid) {
    if (!enabled)
      return null;

    try {

      ItemData data = (ItemData) cache.get(uuid);
      //System.out.println("From cache name "+this+" "+name);
      //if (data != null && data instanceof NodeData)
      //  System.out.println("Cache get by uuid >> " + uuid + " found path: "
      //      + data.getQPath().getAsString() + " primaryType: "
      //      + ((NodeData) data).getPrimaryTypeName().getAsString());

      return data;
    } catch (Exception e) {
      return null;
    }
  }

  /**
   * Called by read operations
   * 
   * @param data
   */
  public void put(ItemData data) {
    try {
      if (enabled && data != null /* && (data instanceof PropertyData) */) {
        cache.put(data.getQPath().getAsString(), data);

        cache.put(data.getUUID(), data);
        // have to refresh cached parent somewhere if use for r/w operations!

        //if (data.isNode())
        //  log.debug("Cache " + name + " put >> "
        //      + data.getQPath().getAsString() + " uuid: " + data.getUUID()
        //      + " child node " + ((NodeData) data).getChildNodes());
      }
    } catch (Exception e) {
    }
  }


  public void clear() throws RepositoryException {
    try {
      log.debug("Cache is about to clear >> " + cache.getCacheSize());
      cache.clearCache();
      log.debug("Cache cleared >> " + cache.getCacheSize());
    } catch (Exception e) {
      e.printStackTrace();
      throw new RepositoryException("Exception in clear Cache " + e);
    }
  }

  public boolean isEnabled() {
    return enabled;
  }

  public void setEnabled(boolean enabled) {
    this.enabled = enabled;
  }

  public void setMaxSize(int maxSize) {
    this.cache.setMaxSize(maxSize);
  }

  public void setLiveTime(long liveTime) {
    this.cache.setLiveTime(liveTime);
  }
  
  // --------------------- ItemsPersistenceListener --------------
  public synchronized void onBatchSaveItems(List itemStates, SessionImpl session) {
  }

  public synchronized void onSaveItems(List itemStates, SessionImpl session) {
    for (Iterator i = itemStates.iterator(); i.hasNext();) {
      ItemState state = (ItemState) i.next();
      ItemData data = state.getData();
      
      try {
        if(state.getState() == ItemChangeState.ADDED) {
          addTree(data);
        } else if(state.getState() == ItemChangeState.DELETED) {
          remove(data);
        } else if(state.getState() == ItemChangeState.UPDATED) {
          update(data);
        }
      } catch (RepositoryException e) {
        e.printStackTrace();
      }
    }
  }
  
  // ---------------------------------------------------

  /**
   * Called by delete
   * @param data
   */
  private void remove(ItemData data) throws RepositoryException {

    if (!enabled)
      return;

    try {
      log.debug("Cache remove item and descendants >> "
          + data.getQPath().getAsString());

      ItemData item = this.get(data.getQPath());
      if (item != null && item instanceof NodeData) {
        NodeData node = (NodeData) item;
        Iterator props = node.getChildProperties().iterator();

        while (props.hasNext()) {
          cache.remove(((ItemData) props.next()).getQPath().getAsString());
        }

        Iterator nodes = node.getChildNodes().iterator();
        while (nodes.hasNext()) {
          cache.remove(((ItemData) nodes.next()).getQPath().getAsString());
        }
      }

      cache.remove(data.getQPath().getAsString());

      cache.remove(data.getUUID());
      //System.out.println("Cache remove >> "+loc.getPath());

    } catch (Exception e) {
      e.printStackTrace();
      throw new RepositoryException("Exception in remove Cache " + e);
    }
  }

  /**
   * Called by add operation
   * 
   * @param data
   */

  private void add(ItemData data) throws RepositoryException {

    if (enabled) {
      put(data);
      
      NodeData parent;
      try {
        parent = (NodeData) get(data.getQPath().makeParentPath()
            .getAsString());
      } catch (IllegalPathException e) {
        throw new RepositoryException(e);
      }
      if (parent != null) {
        log.debug("Cache " + name + " remove parent >> "
            + parent.getQPath().getAsString());
        remove(parent);
      }
    }
  }
  
  private void addTree(ItemData data) throws RepositoryException {
    
    add(data);
    if (data.isNode()) {
      NodeData node = (NodeData) data;
      List props = node.getChildProperties();
      if (props != null)
        for (int i = 0; i < props.size(); i++)
          addTree((PropertyData) props.get(i));
      List nodes = node.getChildNodes();
      if (nodes != null)
        for (int i = 0; i < nodes.size(); i++)
          addTree((NodeData) nodes.get(i));
    }

  }
  
  private void update(ItemData data) {
    if (enabled) {
      put(data);
    }
  }

}
